/* the gmp3 project.
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com/gmp3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "freedb.h"
#include "lib_freedb.h"
#include "gui_freedb.h"
#include "support.h"
#include "interface.h"
#include "mp3info.h"
#include "queue.h"
#include "lib_id3v1.h"
#include "lib_id3v2.3.h"
#include "lib_vorbis.h"
#include "configfile.h"
#include "plugs.h"
#include "shared.h"


/***************************************************************************************
 * BELOW FOLLOW THE STATICS
 ***************************************************************************************/

// Returns the datapointer to the queueitems data with the given fullfilename
static Mp3Info *
g_list_find_matching_fullfilename(GList *haystack, gchar *fullfilename)
{
	GList *haystack_item = NULL;
	Mp3Info *haystack_mp3info = NULL;
	gchar haystack_fullfilename[2048];

	haystack_item = g_list_first(haystack);
	while(haystack_item)
	{
		haystack_mp3info = (Mp3Info *)haystack_item->data;
		haystack_item = g_list_next(haystack_item);
		
		snprintf(haystack_fullfilename, 2047, "%s%s", haystack_mp3info->directory, haystack_mp3info->filename);
		if(strncmp(fullfilename, haystack_fullfilename, 2048)==0)
			return haystack_mp3info;
	}
	return NULL;
}

/***************************************************************************************
 * END OF STATICS
 ***************************************************************************************/






void
freedb_get(void)
{
	extern GtkWidget *gmp3;
	extern GtkWidget *tracks_sort;

	gchar *host_name[50];
	guint host_port[50];
	gchar *cgi_path[50];
	gchar proxy_name[2048];
	guint proxy_port = 0;

	GList *cdlist = NULL;
	
	gchar option[2048];
	gchar value[2048];
	gint cur = 0;
	gchar *config = malloc(10000);
	
// read configfile
	config_read(config, 10000);

// Get list of available FreeDB servers.
	host_name[0] = NULL;
	snprintf(option, 2047, "server0#1");
	while(configfile_option_get(config, "freedb servers", option, value))
	{
		host_name[cur] = strdup(value);
		
		snprintf(option, 2047, "server0#2");
		configfile_option_get(config, "freedb servers", option, value);
		host_port[cur] = (guint)atoi(value);
		
		snprintf(option, 2047, "server0#3");
		configfile_option_get(config, "freedb servers", option, value);

		snprintf(option, 2047, "server0#4");
		configfile_option_get(config, "freedb servers", option, value);
		cgi_path[cur] = strdup(value);
		
		host_name[++cur] = NULL;
		snprintf(option, 2047, "server%i#1", cur);
	}
// get the proxy server
	configfile_option_get(config, "freedb", "proxy address", proxy_name);
	configfile_option_get(config, "freedb", "proxy port", value);
	proxy_port = (guint)atoi(value);
	configfile_option_get(config, "freedb", "enable proxy", value);
	if(strcmp(value, "TRUE")!=0)
		*proxy_name = '\0';
	free(config);
	
	if(!host_name[0])
		return;
	
// get either by freedb id, or by a searchstring.
	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(gmp3, "radio_gmp3_freedb_fulltext"))))
	{
		if(freedb_albumlist_get_http(proxy_name, proxy_port,
				gtk_entry_get_text(GTK_ENTRY(lookup_widget(gmp3, "entry_gmp3_freedb_fulltext_string"))), &cdlist) < 0)
			printf("Error while sending request to server. Using proxy = %s\n", value);
		
		tracks_sort = create_tracks_sort();
		clist_album_choose_albums_add(cdlist);
		gtk_widget_show(tracks_sort);
	}
	else
	{
		CDInfo *cd = malloc(sizeof(CDInfo));
		
		strncpy(cd->genre, gtk_entry_get_text(GTK_ENTRY(lookup_widget(gmp3, "entry_gmp3_freedb_id_category"))), 1023);
		cd->id = strtoul(gtk_entry_get_text(GTK_ENTRY(lookup_widget(gmp3, "entry_gmp3_freedb_id_id"))), 0, 16);
		
		cur = 0;
		while( host_name[cur]!=NULL
			&& freedb_album_get_http(host_name[cur], host_port[cur], cgi_path[cur], proxy_name, proxy_port, &cd)!=0 )
			++cur;
		
// Show the track_sort widget and insert the albums data (=Tracks/Titles).
		tracks_sort = create_tracks_sort();
		clist_tracks_sort_tracks_add(cd);
		gtk_widget_show(tracks_sort);
	}
	
// cleanups
	cur = 0;
	while(host_name[cur])
	{
		free(host_name[cur]);
		free(cgi_path[cur]);
		++cur;
	}
}



void
freedb_album_clicked(void)
{
	extern GtkWidget *tracks_sort;
	gint row = 0;
	
	CDInfo *cd = NULL;
	
	gchar *host_name[50];
	guint host_port[50];
	gchar *cgi_path[50];
	gchar proxy_name[2048];
	guint proxy_port = 0;

	gchar option[2048];
	gchar value[2048];
	gint cur = 0;
	gchar *config = malloc(10000);
	
// read configfile
	config_read(config, 10000);

// Get list of available FreeDB servers.
	host_name[0] = NULL;
	snprintf(option, 2047, "server0#1");
	while(configfile_option_get(config, "freedb servers", option, value))
	{
		host_name[cur] = strdup(value);
		
		snprintf(option, 2047, "server0#2");
		configfile_option_get(config, "freedb servers", option, value);
		host_port[cur] = (guint)atoi(value);
		
		snprintf(option, 2047, "server0#3");
		configfile_option_get(config, "freedb servers", option, value);

		snprintf(option, 2047, "server0#4");
		configfile_option_get(config, "freedb servers", option, value);
		cgi_path[cur] = strdup(value);
		
		host_name[++cur] = NULL;
		snprintf(option, 2047, "server%i#1", cur);
	}
// get the proxy server
	configfile_option_get(config, "freedb", "proxy address", proxy_name);
	configfile_option_get(config, "freedb", "proxy port", value);
	proxy_port = (guint)atoi(value);
	configfile_option_get(config, "freedb", "enable proxy", value);
	if(strcmp(value, "TRUE")!=0)
		*proxy_name = '\0';
	free(config);
	
	if(!host_name[0])
		return;
	
// if none is selected, return.
	if( !(GTK_CLIST(lookup_widget(tracks_sort, "clist_tracks_sort_cdlist"))->selection) )
		return;
	
// Get the chosen albums data
	row = (gint)(GTK_CLIST(lookup_widget(tracks_sort, "clist_tracks_sort_cdlist"))->selection)->data;
	
	cd = (CDInfo *)gtk_clist_get_row_data(GTK_CLIST(lookup_widget(tracks_sort, "clist_tracks_sort_cdlist")), row);
	
	cur = 0;
	while( host_name[cur]!=NULL
		&& freedb_album_get_http(host_name[cur], host_port[cur], cgi_path[cur], proxy_name, proxy_port, &cd)!=0 )
		++cur;
	
// Show the track_sort widget and insert the albums data (=Tracks/Titles).
	gtk_clist_clear(GTK_CLIST(lookup_widget(tracks_sort, "clist_tracks_sort_tracks")));
	clist_tracks_sort_tracks_add(cd);
	
// cleanups
	cur = 0;
	while(host_name[cur])
	{
		free(host_name[cur]);
		free(cgi_path[cur]);
		++cur;
	}
}




void
freedb_tracks_ok(void)
{
	extern GList *queue;
	gchar newfilename[2048];
	gchar newfilename_full[2048];
	CDInfo *cd = NULL;
	gint row = 0;
	gint whichtag = 0;

	id3Tag *tag = malloc(sizeof(id3Tag));
	Mp3Info *mp3info = NULL;
	
	cd = clist_tracks_sort_get_tracks();
	
	if(!cd)
	{
		free(tag);
		return;
	}

	row = -1;
	while(cd->track[++row])
	{
		if( *((CDTrack *)cd->track[row])->filename != '\0' )
		{
			id3Tag *v2 = malloc(sizeof(id3Tag));
			
			mp3info = g_list_find_matching_fullfilename(queue, ((CDTrack *)cd->track[row])->filename);
			
			memset(tag->artist, '\0', 1024);
			memset(tag->title, '\0', 1024);
			memset(tag->album, '\0', 1024);
			memset(tag->year, '\0', 5);
			memset(tag->track, '\0', 3);
			memset(tag->comment, '\0', 1024);

// Get old Tag
			if( strtoucmp(((CDTrack *)cd->track[row])->filename + strlen(((CDTrack *)cd->track[row])->filename)-4, ".ogg") == 0 )
			{
#ifdef HAVE_OGG_H
				get_vorbis_tag(tag, ((CDTrack *)cd->track[row])->filename);
				whichtag = 3;
#endif
			}
			else
			{
// get the tag to a struct.
				get_id3v1_tag(tag, ((CDTrack *)cd->track[row])->filename);

// v2 tag has higher prio, so it will overwrite the v1	
				if(get_id3v2_tag(v2, ((CDTrack *)cd->track[row])->filename)==0)
				{
					v2->genre = tag->genre;
					free(tag);
					tag = v2;
				}
				else
					free(v2);
				
				whichtag = 1;
			}

			
// And put the changed tag to the file
			if(access(((CDTrack *)cd->track[row])->filename, W_OK)==0)
			{
// the cdtitle can have two different formats in freedb:
// either "Artist / Album" or only "Sampler" or stuff.
// so we look for an " / ", if its there, we split it into artist and cdtitle
// otherwise we set both the same, artist and cdtitle.
// copy freedb album title to tag
				if(strstr(cd->cdtitle, " / ")!=NULL)
				{
					strncpy(tag->artist, cd->cdtitle, 1024);
					*(strstr(tag->artist, " / ")) = '\0';
					strncpy(tag->album, strstr(cd->cdtitle, " / ")+3, 1024);
				}
				else
				{
					strncpy(tag->artist, cd->cdtitle, 1024);
					strncpy(tag->album, cd->cdtitle, 1024);
				}

// copy track title to tag
// all the same as the cdtitle, we have the two possible formats.
				if(strstr(((CDTrack *)cd->track[row])->title, " / ")!=NULL)
				{
					strncpy(tag->artist, ((CDTrack *)cd->track[row])->title, 1024);
					*(strstr(tag->artist, " / ")) = '\0';
					strncpy(tag->title, strstr(((CDTrack *)cd->track[row])->title, " / ")+3, 1024);
				}
				else
					strncpy(tag->title, ((CDTrack *)cd->track[row])->title, 1024);
				
// track number and year.
				snprintf(tag->track, 3, "%i", row+1);
				strncpy(tag->year, cd->year, 5);

// save tags.
				if( whichtag==1
					&& *(((CDTrack *)cd->track[row])->filename)!='\0' )
				{
					set_id3v1_tag(tag, ((CDTrack *)cd->track[row])->filename);
				
					if(set_id3v2_tag(tag, ((CDTrack *)cd->track[row])->filename)==0)
					{
// after we set the tag, we re-read its tag information.
						mp3file_get_info_1(mp3info->directory, mp3info->filename, mp3info);
// I then create the new filename.
						snprintf(newfilename, 2047, "%i - %s.mp3", row+1, ((CDTrack *)cd->track[row])->title);
						apply_rule_replace_invalid_characters(newfilename);
// and the new fullfilename
						strncpy(newfilename_full, ((CDTrack *)cd->track[row])->filename, 2047);
						*(strrchr(newfilename_full, '/')+1) = '\0';
						strncpy( newfilename_full+strlen(newfilename_full), newfilename, 2047-strlen(newfilename_full) );
// just to be sure we do nothing stupid we replace a " / " with a " - "
						if(strstr(newfilename_full, " / ") != NULL)
							*(strstr(newfilename_full, " / ")+1) = '-';

// be sure we don't overwrite any old file
						if( *(((CDTrack *)cd->track[row])->filename) != '\0'
							&& access(newfilename_full, F_OK)==0 )
						{
							strncpy(mp3info->status, "File exists!", 255);
						}
						else
						{
// Rename file
							if(rename(((CDTrack *)cd->track[row])->filename, newfilename_full)==0)
							{
								strncpy(mp3info->filename, newfilename, 2047);
								strncpy(mp3info->status, "FreeDB OK!", 255);
							}
							else
								strncpy(mp3info->status, "Tags set, rename failed!", 255);
						}
					}
					else
						strncpy(mp3info->status, "Failed!", 255);
				}
#ifdef HAVE_OGG_H
// The same for Ogg.
				else
				{
					if( set_vorbis_tag(tag, ((CDTrack *)cd->track[row])->filename)==0 )
					{
// after we set the tag, we re-read its tag information.
						mp3file_get_info_1(mp3info->directory, mp3info->filename, mp3info);
// I then create the new filename.
						snprintf(newfilename, 2047, "%i - %s.ogg", row+1, ((CDTrack *)cd->track[row])->title);
						apply_rule_replace_invalid_characters(newfilename);
// and the new fullfilename
						strncpy(newfilename_full, ((CDTrack *)cd->track[row])->filename, 2047);
						*(strrchr(newfilename_full, '/')+1) = '\0';
						strncpy( newfilename_full+strlen(newfilename_full), newfilename, 2047-strlen(newfilename_full) );
// just to be sure we do nothing stupid we replace a " / " with a " - "
						if(strstr(newfilename_full, " / ") != NULL)
							*(strstr(newfilename_full, " / ")+1) = '-';

// be sure we don't overwrite any old file
						if( *(((CDTrack *)cd->track[row])->filename) != '\0'
							&& access(newfilename_full, F_OK)==0 )
						{
							strncpy(mp3info->status, "File exists!", 255);
						}
						else
						{
// Rename file
							if(rename(((CDTrack *)cd->track[row])->filename, newfilename_full)==0)
							{
								strncpy(mp3info->filename, newfilename, 2047);
								strncpy(mp3info->status, "FreeDB OK!", 255);
							}
							else
								strncpy(mp3info->status, "Tags set, rename failed!", 255);
						}
					}
					else
						strncpy(mp3info->status, "Failed!", 255);
				}
#endif
			}
			else
			{
// ok, we don't have write access
				if( *(((CDTrack *)cd->track[row])->filename)!='\0'
					&& access(newfilename_full, W_OK)!=0 )
				{
					strncpy(mp3info->status, "Locked!", 255);
				}
				else
					strncpy(mp3info->status, "Unchanged!", 255);
			}
		}
	}

// cleanups
	free(tag);
	row = -1;
	while(cd->track[++row])
		free(cd->track[row]);
	free(cd);
}





void
freedb_tracks_cancel(void)
{
	extern GtkWidget *tracks_sort;
	CDInfo *cd = NULL;
	gint row = 0;

	cd = clist_tracks_sort_get_tracks();
	
	gtk_widget_destroy(tracks_sort);
	
	if(!cd)
		return;

	row = -1;
	while(cd->track[++row])
		free(cd->track[row]);
	free(cd);
}
