/* the gmp3 project.
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com/gmp3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <dirent.h>
#include <sys/stat.h>
#include "memory.h"
#include "gui_dirtree.h"

/***************************************************************************************
 * BELOW FOLLOW THE STATICS
 ***************************************************************************************/

// TRUE if directory, FALSE if not.
static int
is_dir(gchar *filepathname)
{
	struct stat pstat;
	stat(filepathname, &pstat);
	return(S_ISDIR(pstat.st_mode));
}

/***************************************************************************************
 * END OF STATICS
 ***************************************************************************************/





/**********************************************************************
 * Add the rootdir to the directory-ctree
 * Arguments: ctree: The ctree to put this rootdir in.
 *            data: A glist with pointers to alloced data
 **********************************************************************/
void
ctree_init_dirtree(GtkCTree *ctree, GList *data)
{
	extern GtkWidget *gmp3;
	
	GtkCTreeNode *parentnode = NULL;
	GtkCTreeNode *dummynode = NULL;
	gchar *column[1];
	gchar *temp = NULL;

	GdkColormap *colormap = NULL;
	GdkPixmap *pixmap_close = NULL;
	GdkPixmap *pixmap_open = NULL;
	GdkBitmap *mask_close = NULL;
	GdkBitmap *mask_open = NULL;

	colormap = gtk_widget_get_colormap (gmp3);
	
	gtk_ctree_set_indent(ctree, 10);
	setenv("PWD", "/", 1);

// Init parent node ("/")
	column[0] = mem_allocate(data, 2);
	sprintf(column[0], "/");

// Init the pixmap for closed branches
	temp = g_strconcat(PACKAGE_PIXMAPS_DIR, "/folder_closed.xpm", NULL);
	pixmap_close = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask_close, NULL, temp);
	free(temp);
// Init the pixmap for opened branches
	temp = g_strconcat(PACKAGE_PIXMAPS_DIR, "/folder_opened.xpm", NULL);
	pixmap_open = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask_open, NULL, temp);
	free(temp);

// Insert parentnode into ctree
	parentnode = gtk_ctree_insert_node(ctree, NULL, NULL, column, 5, pixmap_close, mask_close, pixmap_open, mask_open, FALSE, FALSE);
	gtk_ctree_node_set_row_data(ctree, parentnode, column[0]);

// and insert one dummy node, so the "+" appears.
	column[0] = mem_allocate(data, 2);
	sprintf(column[0], "*");
	dummynode = gtk_ctree_insert_node(ctree, parentnode, NULL, column, 0, NULL, NULL, NULL, NULL, FALSE, FALSE);
	gtk_ctree_node_set_row_data(ctree, dummynode, (gpointer) "*");

// we just open the new created "/"-node, that will produce the signal that will add more nodes.
	gtk_ctree_expand(ctree, parentnode);
}






/**********************************************************************
 * Add a branch to the directory tree
 * Arguments: ctree: the ctree in which to expand the branch.
 *            row:   the row number of the item to expand.
 *            data:  A glist with pointers to alloced data
 **********************************************************************/
void
ctree_expand_dirtree(GtkCTree *ctree, gint row, GList *data, gboolean showhidden)
{
	extern GtkWidget *gmp3;
	
	GtkCTreeNode *node = NULL;
	GtkCTreeNode *subnode = NULL;
	GtkCTreeNode *dummynode = NULL;
	gchar *column[1];

	struct dirent *pdirent = NULL;
	DIR *dirstream = NULL, *subdirstream = NULL;

	gchar *directory = NULL;
	gchar *subdirectory = NULL;
	gchar filepathname[2048];
	gchar *temp = NULL;

	GdkColormap *colormap = NULL;
	GdkPixmap *pixmap_opened = NULL;
	GdkPixmap *pixmap_closed = NULL;
	GdkPixmap *pixmap_locked = NULL;
	GdkBitmap *mask_opened = NULL;
	GdkBitmap *mask_closed = NULL;
	GdkBitmap *mask_locked = NULL;

	colormap = gtk_widget_get_colormap(gmp3);

// Delete dummy entry (if present). This will collapse the expanded branch!!
// if no dummy entry is present, then there are no subdirectories, so return.
	node = gtk_ctree_node_nth(ctree, row + 1);
	directory = (char *)gtk_ctree_node_get_row_data(ctree, node);
	if(*directory=='*')
		gtk_ctree_remove_node(ctree, node);
	else
		return;

	gtk_clist_freeze(GTK_CLIST(ctree));

// Then, get the directory name, which is attached to the node as data
 	node = gtk_ctree_node_nth(ctree, row);
	directory = (char *)gtk_ctree_node_get_row_data(ctree, node);

// open directory or fatal error.
	if ((dirstream = opendir (directory)) == NULL) 
	{
		printf("Error opening requested path ('%s')!\n", directory);
		exit(1);
	}

// prepare the opened pixmap
	temp = g_strconcat(PACKAGE_PIXMAPS_DIR, "/folder_opened.xpm", NULL);
	pixmap_opened = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask_opened, NULL, temp);
	free(temp);
// prepare the closed pixmap
	temp = g_strconcat(PACKAGE_PIXMAPS_DIR, "/folder_closed.xpm", NULL);
	pixmap_closed = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask_closed, NULL, temp);
	free(temp);
// prepare the locked pixmap
	temp = g_strconcat(PACKAGE_PIXMAPS_DIR, "/folder_locked.xpm", NULL);
	pixmap_locked = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask_locked, NULL, temp);
	free(temp);

// Add each child-dir to the node.
	while ((pdirent = readdir(dirstream)) != NULL)
	{
// Get next name from stream and create a fullpath.
		subdirectory = mem_allocate(data, 2048);
		sprintf(subdirectory, "%s%s", directory, pdirent->d_name);
		if( is_dir(subdirectory)
		    && memcmp(pdirent->d_name, "..\0", 3)
		    && memcmp(pdirent->d_name, ".\0", 2)
			&& ( (strncmp(pdirent->d_name, ".", 1) != 0) || showhidden ) )
		{
// If this is a valid directory, we allocate mem for the tree entry and store the dirname there.
			column[0] = mem_allocate(data, 2048);
			strncpy(column[0], pdirent->d_name, 2047);
// If we have no read access for the file, we must place an alternate pixmap there.
			if(access(subdirectory, R_OK) == 0)
				subnode = gtk_ctree_insert_node(GTK_CTREE (ctree), node, NULL, column, 5, pixmap_closed, mask_closed, pixmap_opened, mask_opened, FALSE, FALSE);
			else
				subnode = gtk_ctree_insert_node(GTK_CTREE (ctree), node, NULL, column, 5, pixmap_locked, mask_locked, pixmap_locked, mask_locked, FALSE, FALSE);
// now we append the allocated mem containing the dirname to the newmade node (with an added slash).
			strncpy(subdirectory+strlen(subdirectory), "/\0", 2);
			gtk_ctree_node_set_row_data(ctree, subnode, subdirectory);
			
// so the childnode is inserted now, look for the childs subdirs. if any subdirs exist, enable the "+" box by adding a dummy-subnode.
			if((subdirstream = opendir(subdirectory)) != NULL)
			{
				dummynode = NULL;
				while ((pdirent = readdir(subdirstream)) != NULL)
				{
					sprintf (filepathname, "%s%s", subdirectory, pdirent->d_name);
					if ( is_dir(filepathname)
					    && memcmp(pdirent->d_name, "..\0", 3)
		    			&& memcmp(pdirent->d_name, ".\0", 2) )
					{
						dummynode = gtk_ctree_insert_node(GTK_CTREE (ctree), subnode, NULL, column, 0, NULL, NULL, NULL, NULL, FALSE, FALSE);
						gtk_ctree_node_set_row_data(ctree, dummynode, (gpointer)"*");
						break;
					}
				}
			}
			closedir(subdirstream);
		}
	}
	closedir(dirstream);

// Cleanup...
	gtk_ctree_sort_recursive(ctree, node);
	gtk_ctree_expand(ctree, node);
	gtk_clist_thaw(GTK_CLIST(ctree));
}









/**********************************************************************
 * Remove a branch of the directory tree
 * Arguments: ctree: the ctree in which to collapse a branch.
 *            row:   the row number of the item to colapse.
 *            data:  A glist with pointers to alloced data
 **********************************************************************/
void
ctree_collapse_dirtree(GtkCTree *ctree, gint row, GList *data)
{
	GtkCTreeNode *node = NULL;
	GtkCTreeNode *child = NULL;
	GtkCTreeNode *dummynode = NULL;
	gchar *column[1];

	gtk_clist_freeze(GTK_CLIST(ctree));
	node = gtk_ctree_node_nth(ctree, row);
	child = GTK_CTREE_ROW(node)->children;
	
// remove all children
	while(child)
	{
		data = mem_free_1( data,
			gtk_ctree_node_get_row_data(ctree, child) );
		gtk_ctree_remove_node(ctree, child);
		child = GTK_CTREE_ROW(node)->children;
	}

// and add a dummy node again.
	column[0] = calloc(2,2);
	*column[0] = '*';
	dummynode = gtk_ctree_insert_node(GTK_CTREE (ctree), node, NULL, column, 0, NULL, NULL, NULL, NULL, FALSE, FALSE);
	gtk_ctree_node_set_row_data(ctree, dummynode, (gpointer) "*");
	free(column[0]);
	
	gtk_clist_thaw(GTK_CLIST(ctree));
}




void
ctree_dirtree_open_directory(GtkCTree *ctree, gchar *directory)
{
	GtkCTreeNode *node = NULL;
	gchar *pdirectory = NULL;
	gchar subdir[2048] = "\0";
	gchar curdir[4096] = "\0";
	gchar curdircpy[4096] = "\0";
	gint i = -1;
	GList *dirs = NULL;
	
	if(!directory)
		return;
	
// append a slash, if there is none yet.
	if(*(directory+strlen(directory)-1) != '/')
		*(directory+strlen(directory)) = '/';
	
	gtk_clist_freeze(GTK_CLIST(ctree));
	pdirectory = directory;
// make a list of all diectories ill have to open, e.g. for /home/sam it mus be a list of "/", "/home/" and "/home/sam/"
	while(strchr(pdirectory, '/'))
	{
		strncpy(subdir, pdirectory, 2047);
		*(strchr(subdir, '/') + 1) = '\0';
		pdirectory = strchr(pdirectory, '/') + 1;
		
		strncpy(curdircpy, curdir, 4096);
		snprintf(curdir, 4096, "%s%s", curdircpy, subdir);
		dirs = g_list_append(dirs, strdup(curdir));
	}
	
	while(dirs)
	{
		i = -1;
		while( (node = gtk_ctree_node_nth(ctree, ++i)) )
		{
			if(strcmp((gchar *)dirs->data, gtk_ctree_node_get_row_data(ctree, node)) == 0)
			{
				gtk_ctree_expand(ctree, node);
// if this is the last directory, we select it in the ctree
				if(!dirs->next)
				{
					gtk_ctree_select(ctree, node);
					gtk_ctree_node_moveto(ctree, node, 0, 0.3, 0);
				}
			}
		}
		free(dirs->data);
		dirs = dirs->next;
	}
	gtk_clist_thaw(GTK_CLIST(ctree));
}




