/* the gmp3 project.
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com/gmp3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <gnome.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include "lib_freedb.h"


/***************************************************************************************
 * BELOW FOLLOW THE STATICS
 ***************************************************************************************/

// receive one line from a host
static gint
get_line(gint socket, gchar *line, gint maxlen)
{
	gint i = 0;
	gint foo = 0;
	
// We need to get char by char, to the first occurence of a line feed, so we get only one line.
	memset(line, 0, maxlen);
	for(i=0; i < maxlen; i++)
	{
		if( strlen(line) > 2
			&& *(line + i - 1) == '\n' )
			break;
		foo = read(socket, line + i, 1);
	}
	
// remove the linefeed.
	while( (*(line + strlen(line) - 1) == '\n' || *(line + strlen(line) - 1) == '\r')
		&& strlen(line) > 0 )
		*(line + strlen(line) - 1) = 0;
	
	return foo;
}

/***************************************************************************************
 * END OF STATICS
 ***************************************************************************************/





gint
freedb_album_get_http(gchar *hostname, guint host_port, gchar *cgi_path, gchar *proxy, guint proxy_port, CDInfo **cd)
{
	struct sockaddr_in socket_address;
	struct hostent *hostent;
	gint socket_number;

	gchar bin[4096];		// Binary in data
	gchar bout[4096];		// Binary out data
	
	CDTrack *track;
	
	(*cd)->track[0] = NULL;

// Get a Socket number
	if((socket_number = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
		perror("Error: Can't open INET socket!");
		return -1;
	}
	
// No proxy? Then the host is proxy, also.
	if(!proxy
		|| *proxy=='\0')
	{
		proxy = hostname;
		proxy_port = host_port;
	}
	
// Server address lookup
	if( !(hostent = gethostbyname(proxy)) )
	{
		perror("Failed to lookup the server!");
		return -2;
	}
	
	socket_address.sin_family = AF_INET;
	memcpy(&socket_address.sin_addr, hostent->h_addr, hostent->h_length);
	socket_address.sin_port = htons(proxy_port);

// Connect to the server...
	if(connect(socket_number, (struct sockaddr *)&socket_address, sizeof(socket_address)) == -1)
	{
		perror("Can't connect to FreeDB host!");
		return -3;
	}
	
// Send request
	sprintf( bout, "GET http://%s%s?cmd=cddb+read+%s+%08lx&hello=somebody@linux.com+chef+gmp3+0.8&proto=1 HTTP/1.1\r\nConnection: close\nHost: %s:%i\r\n\r\n",
		hostname,
		cgi_path,
		(*cd)->genre,
		(*cd)->id,
		hostname,
		host_port );
	if( write(socket_number, bout, strlen(bout)) < 0 )
	{
		perror("Error while sending request to the server!");
		close(socket_number);
		return -4;
	}

// We get the answer of the query
// the first line does NOT contain a title (and there may be more lines like that), but some header informations
// we dont care about.
	if( get_line(socket_number, bin, sizeof(bin)) <= 0 )
	{
		perror("Error while receiving request from the server!");
		close(socket_number);
		return -5;
	}
	if( strncmp(bin, "HTTP/1.1 200 ", 13) != 0 )
	{
		printf("Error while receiving track information. (%s)\n", bin);
		close(socket_number);
		return -6;
	}

// so the server sent a valid reply, we can get all titles now.
// we only care about lines beginning with "TTITLE", that will skip all header lines and stuff.
// a line containing nothing but a dot (plus a linefeed) means end of data.
	while(strcmp(bin, ".\r\n") != 0)
	{
// Get one line
		if( get_line(socket_number, bin, sizeof(bin)) <= 0 )
		{
			if( !(*cd)->track[0] )
			{
// If we received en empty line before we receive a track, return with an error.
				printf("Error while receiving track information!");
				close(socket_number);
				return -7;
			}
			else
				break;
		}

		if(!strncmp(bin, "TTITLE", 6))
		{
			track = malloc(sizeof(CDTrack));
			
// the line has the format: "TTITLEnn=<title>"
// so we save anything after the "=" as tracktitle, and the number between "TTITLE" and the "=" as track.
			strncpy(track->number, bin + 6, 19);
			if( strchr(track->number, '=') )
				*strchr(track->number, '=') = '\0';
			
			if( strchr(bin, '=') )
				strncpy(track->title, strchr(bin, '=') + 1, 2048);
			
// it is possible to split a track name via multiple freedb lines.
// thus, the same track may have occured before. In that case, we just append the current line to the older one.
			if( (*cd)->track[atoi(track->number)] )
			{
// we must be aware of the maxlen of 2048.
				strncpy(
					((CDTrack *)(*cd)->track[atoi(track->number)])->title + strlen( ((CDTrack *)(*cd)->track[atoi(track->number)])->title ),
					track->title,
					2048 - strlen( ((CDTrack *)(*cd)->track[atoi(track->number)])->title ) );
				
				free(track);
			}
			else
			{
				((CDTrack *)(*cd)->track[atoi(track->number)]) = track;
				(*cd)->track[atoi(track->number) + 1] = NULL;
			}
		}

		if(strncmp(bin, "DYEAR", 5) == 0)
		{
// the line has the format: "DYEAR=<title>"
// so we save anything after the "=" as year.
			if( strchr(bin, '=') )
				strncpy((*cd)->year, strchr(bin, '=')+1, 4);
		}
	}

// Close the socket connection with my host
	close(socket_number);
	return 0;
}










gint
freedb_albumlist_get_http(gchar *proxy, guint proxy_port, gchar *pstring, GList **cdlist)
{
	struct sockaddr_in socket_address;
	struct hostent *hostent;
	gint socket_number = 0;

	gchar bin[4096];		// Binary in data
	gchar bout[14096];		// Binary out data
	gchar buf[4096];
	gchar string[4096];
	gchar hostname[1024] = "www.freedb.org";
	gint host_port = 80;
	gchar *temp = NULL;
	
	gboolean have = FALSE;
	gint i = 0;
	CDInfo *cd = NULL;

	if(!pstring
		|| *pstring=='\0')
		return 0;
	
	strncpy(string, pstring, sizeof(string));
// Change all spaces in the searchstring to "+" (URLsafe)
	while(strchr(string, ' ')!=NULL)
		*strchr(string, ' ') = '+';
	
// Get a Socket number
	if((socket_number = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
		perror("Error: Can't open INET socket!");
		return -1;
	}

// No proxy? Then the host is proxy, also.
	if(!proxy
		|| *proxy=='\0')
	{
		proxy = hostname;
		proxy_port = host_port;
	}
	
// Server lookup
	if( !(hostent = gethostbyname(proxy)) )
	{
		perror("Failed to lookup the server!");
		return -2;
	}
	
	socket_address.sin_family = AF_INET;
	memcpy(&socket_address.sin_addr, hostent->h_addr, hostent->h_length);
	socket_address.sin_port = htons(proxy_port);

// Connect to the server...
	if(connect(socket_number, (struct sockaddr *)&socket_address, sizeof(socket_address)) == -1)
	{
		perror("Can't connect to FreeDB host!");
		return -3;
	}
	
// Send request
	sprintf(bout, "GET http://%s/freedb_search.php?words=%s&allfields=YES&fields=artist&fields=title&allcats=YES&grouping=none HTTP/1.1\nConnection: close\nHost: %s:%i\n\n",
		hostname,
		string,
		hostname,
		host_port);
	if( write(socket_number, bout, strlen(bout)) < 0 ) {
		perror("Error while sending request to the server!\n");
		close(socket_number);
		return -4;
	}

// Ok! i now read the answer
	i = 0;
	have = FALSE;
	while(strncmp(bin, "</body>", 7)!=0)
	{
// Get one line
		if( get_line(socket_number, bin, sizeof(bin)) <= 0 )
		{
			printf("Error while receiving cd information! (have = %i)\n", have);
			close(socket_number);
			return -7;
		}
		
		if( strncmp(bin, "<tr><td><a href=\"http://www.freedb.org/freedb_search_fmt.php?cat=", 65)==0 )
		{
// ok, we have found one line that looks like a cd title in the html output.
			cd = malloc(sizeof(CDInfo));
			
// the cd title should begin after char nr. 64 and end at the first occurence of "</a><br><"
// so we strip all unneeded chars at the string end here.
			temp = &bin[64];
			while( *(++temp) != '\0'
				&& *temp != '\n' )
			{
				if(strncmp(temp, "</a><br><", 9) == 0)
				{
					*temp = '\0';
					break;
				}
			}
			
// maxlen 1024.
			strncpy(cd->cdtitle, strchr(&bin[65], '>')+1, 1023);

// the genre begins at char 65
			strncpy(cd->genre, &bin[65], 1023);
			if( strchr(cd->genre, '&') )
				*strchr(cd->genre, '&') = '\0';
			else
			{
				perror("Wrong format of the freedb output! Contact this programs author if this happens often.");
				close(socket_number);
				return -8;
			}

// the freedb id begins after the first "=" after char nr. 65
			if( strchr(&bin[65], '=')
				&& strchr(strchr(&bin[65], '='), '\"') )
			{
				strncpy(buf, strchr(&bin[65], '=')+1, sizeof(buf));
				*strchr(buf, '\"') = '\0';
				cd->id = strtoul(buf, NULL, 16);
			}
			else
			{
				perror("Wrong format of the freedb output! Contact this programs author if this happens often.");
				close(socket_number);
				return -9;
			}
			
			*cdlist = g_list_append(*cdlist, cd);
			have = TRUE;
		}
		else if( strncmp(bin, "<br><br></tr>", 13) != 0
				&& (have==TRUE) )
		{
			break;
		}
	}

// Close the socket connection with my host
	close(socket_number);
	return 0;
}
