/* the gmp3 project.
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com/gmp3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gnome.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <ctype.h>
#include "mp3info.h"
#include "lib_id3v1.h"

/*
 * Purpose: Reads the ID3 tag from a file.
 * Parameters: tag - The structure to store the tag in, filename - The name of the file to operate on.
 * Returns:
 * 0 if successful,
 * 1 if an error occured when opening the file
 * 2 if error while reading tag.
 * 3 if no TAG found.
 */
int get_id3v1_tag(id3Tag *tag, gchar *filename)
{
	FILE *mp3file;
	gchar *buffer = calloc(2000, sizeof(gchar));
	id3v1Tag *v1 = calloc(1, sizeof(id3v1Tag));
	gboolean foundtag = FALSE;
	gint i = -1;

	memset(tag, 0, sizeof(id3Tag));

	mp3file = fopen(filename, "rb");
	if (!mp3file) return(1);

// read the last 400 bytes of the file.
// in most cases, the last 128 bytes would be enough, but some buggy tags, again...
	fseek(mp3file, -400, SEEK_END);
	if (!fread(buffer, 1, 400, mp3file))
	{
		free(buffer);
		free(v1);
		fclose(mp3file);
		return (2);
	}
	fclose(mp3file);

	while(++i <= (400-128) )
	{
		if( (*(buffer+i+0)=='T')
			&& (*(buffer+i+1)=='A')
			&& (*(buffer+i+2)=='G') )
		{
			memcpy(v1, buffer+i+3, 128);
			foundtag = TRUE;
			break;
		}
	}
	free(buffer);
	if(!foundtag) {
		free(v1);
		return(3);
	}

	strncpy(tag->title, v1->title, 30);
	strncpy(tag->artist, v1->artist, 30);
	strncpy(tag->album, v1->album, 30);
	strncpy(tag->year, v1->year, 4);
	if(v1->comment[28]=='\0' && v1->comment[29]!='\0')
	{
		strncpy(tag->comment, v1->comment, 28);
		snprintf(tag->track, 3, "%i", v1->comment[29]);
		tag->comment[29] = '\0';
	}
	else
	{
		strncpy(tag->comment, v1->comment, 30);
		tag->comment[30] = '\0';
	}
	tag->genre = v1->genre;

	free(v1);
	return (0);
}



/*
 * Purpose: Writes the ID3 tag to the file.
 * Parameters: tag - The tag to write, filename - The name of the file to operate on.
 * Returns:
 * 0 if successful,
 * 1 if an error occured when opening the file for reading
 * 2 if error while reading whole file.
 * 3 if an error occured while writing the whole file
 * 4 if error while opening file for tagadding.
 * 5 if error while writing the tag.
 */
int set_id3v1_tag(id3Tag *tag, gchar *filename)
{
	FILE *mp3file;
	id3v1Tag *v1;

	mp3file = fopen(filename, "r+b");
	if (!mp3file) return (1);

// if there is an existing tag, delete it.
	fseek(mp3file, -128, SEEK_END);
	if (fgetc(mp3file)=='T' && fgetc(mp3file)=='A' && fgetc(mp3file)=='G')
	{
		fclose(mp3file);
		del_id3v1_tag(filename);
	}

// Prepare users data for writing
	v1  = calloc(2000,sizeof(id3v1Tag));
	memcpy(v1->title, tag->title, 30);
	memcpy(v1->artist, tag->artist, 30);
	memcpy(v1->album, tag->album, 30);
	memcpy(v1->year, tag->year, 4);
	if(tag->track)
	{
		memcpy(v1->comment, tag->comment, 28);
		v1->comment[28]='\0';
		*(v1->comment+29) = atoi(tag->track);
	}
	else
	{
		memcpy(v1->comment, tag->comment, 30);
	}

	v1->genre = tag->genre;

	mp3file = fopen(filename, "r+b");
	if (!mp3file) return (4);
// now add the new tag
	fseek(mp3file, 0, SEEK_END);
	fputc('T', mp3file);
	fputc('A', mp3file);
	fputc('G', mp3file);
	
	if (!fwrite(v1, sizeof(id3v1Tag), 1, mp3file))
	{
		fclose(mp3file);
		return (5);
	}
	
	fclose(mp3file);

	free(v1);
	return (0);
}





/*
 * Name says it all.
 * Returns:
 * 0 if successful,
 * 1 if an error occured when opening the file
 * 2 if error while reading file.
 * 3 if no TAG found.
 */
int del_id3v1_tag(gchar *filename)
{
	FILE *mp3stream;
	gint mp3file;
	long i=0, len;
	gchar *buffer = calloc(2000, sizeof(gchar));

	mp3stream = fopen(filename, "r+b");
	fseek(mp3stream, 0, SEEK_END);
	len = ftell(mp3stream);
	fclose(mp3stream);
	
	mp3file = open(filename, O_RDWR);
	if (mp3file==-1) return (1);

// if there is an existing tag, delete it.
// read the last 400 bytes of the file.
// in most cases, the last 128 bytes would be enough, but some buggy tags, again...
	lseek(mp3file, -400L, SEEK_END);
	if (read(mp3file, buffer, 400) < 400)
	{
		free(buffer);
		close(mp3file);
		return (2);
	}
	while(++i <= 400)
	{
		if( (*(buffer+i+0)=='T')
			&& (*(buffer+i+1)=='A')
			&& (*(buffer+i+2)=='G') )
		{
			ftruncate(mp3file, len-i);
			break;
		}
	}
	free(buffer);
	close(mp3file);
	return (0);
}
