/* the gmp3 project.
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com/gmp3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef HAVE_OGG_H

#include <gnome.h>
#include <vorbis/vorbisfile.h>
#include <vorbis/codec.h>
#include "lib_vorbis.h"
#include "charset.h"
#include "vcedit.h"


/***************************************************************************************
 * BELOW FOLLOW THE STATICS
 ***************************************************************************************/

/* Stolen from XMMS (and modified heavily)
 *  XMMS - Cross-platform multimedia player
 *  Copyright (C) 1998-2001  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *  Copyright (C) 1999-2001  Hvard Kvlen
 *  Copyright (C) 2001, Jorn Baayen <jorn@nl.linux.org>
 */
static gint
Ogg_Tag_Write_File (FILE *file_in, gchar *filename_in, vcedit_state *state)
{
    gchar *filename_tmp;
    FILE  *file_out;
    gint return_code = 0;


    filename_tmp = g_strdup_printf("%s.tempXXXXX", filename_in);

    if ( (file_out=fopen(filename_tmp,"wb")) == NULL )
    {
        fclose(file_out);
        remove(filename_tmp);
        g_free(filename_tmp);
        fclose(file_in);
        return 2;
    }

    if(vcedit_write(state, file_out) < 0)
        return_code = 3;

    fclose(file_in);

    if (fclose(file_out) != 0)
        return_code = 4;

    if ( (return_code!=0) || (rename(filename_tmp, filename_in)<0) )
    {
        remove(filename_tmp);
        return_code = 5;
    }

    g_free(filename_tmp);

    return return_code;
}

/***************************************************************************************
 * END OF STATICS
 ***************************************************************************************/




int
get_vorbis_tag(id3Tag *tag, gchar *filename)
{
	FILE *oggfile;
	OggVorbis_File vf;
	int i = 0;
	gchar *temp = NULL;

// Open file
	oggfile = fopen(filename, "rb");
	if (!oggfile)
		return(1);

// Pipe the file to libogg
	if(ov_open(oggfile, &vf, NULL, 0) < 0)
	{
		return(2);
	}

// Parse the comments, and put them into the tag struct
	{
		char **ptr = ov_comment(&vf, -1)->user_comments;
//		vorbis_info *vi = ov_info(&vf, -1);
		
		while(*ptr)
		{
			temp = convert_from_utf8(*ptr);
			if(strchr(temp, '='))
			{
				if( strncmp(temp, "TITLE", 5) == 0
					|| strncmp(*ptr, "title", 5) == 0 )
					strncpy(tag->title, strchr(temp, '=')+1, 1024);
				
				if( strncmp(temp, "ARTIST", 6) == 0
					|| strncmp(temp, "artist", 6) == 0 )
					strncpy(tag->artist, strchr(temp, '=')+1, 1024);
				
				if( strncmp(temp, "ALBUM", 5) == 0
					|| strncmp(temp, "album", 5) == 0 )
					strncpy(tag->album, strchr(temp, '=')+1, 1024);
				
				if( strncmp(temp, "DATE", 4) == 0
					|| strncmp(temp, "date", 4) == 0
					|| strncmp(temp, "year", 4) == 0 )
					strncpy(tag->year, strchr(temp, '=')+1, 4);
				
				if( strncmp(temp, "DESCRIPTION", 11) == 0
					|| strncmp(temp, "comment", 7) == 0 )
					strncpy(tag->comment, strchr(temp, '=')+1, 1024);
				
				if( strncmp(temp, "TRACKNUMBER", 11) == 0
					|| strncmp(temp, "tracknumber", 11) == 0 )
					strncpy(tag->track, strchr(temp, '=')+1, 2);
				
				if( strncmp(temp, "GENRE", 5) == 0
					|| strncmp(temp, "genre", 5) == 0 )
				{
					for(i=0; i<=147; i++)
					{
						if( strcmp(genrelist[i], strchr(temp, '=')+1) == 0 )
							tag->genre = i;
					}
				}
			}
			free(temp);
			
			++ptr;
		}
	}
	
// cleanup
	ov_clear(&vf);
	return(0);
}




/* Stolen from XMMS.
 *  Copyright (C) 1998-2001  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *  Copyright (C) 1999-2001  Hvard Kvlen
 *  Copyright (C) 2001, Jorn Baayen <jorn@nl.linux.org>
 */
int
set_vorbis_tag(id3Tag *tag, gchar *filename)
{
	FILE *oggfile;
    vcedit_state *state;
    gchar *string, *string1;
    vorbis_comment *vc;
	gint i = 0;

// Open file
	oggfile = fopen(filename, "rb");
	if (!oggfile)
		return(1);

    state = vcedit_new_state();    // Allocate memory for 'state'
    if ( vcedit_open(state, oggfile) < 0 )
    {
        g_print("ERROR: Failed to open file: '%s' as vorbis (%s).\n", filename, vcedit_error(state));
        fclose(oggfile);
        return(2);
    }

// Get data from tag
    vc = vcedit_comments(state);
    vorbis_comment_clear(vc);
    vorbis_comment_init(vc);

// Title
    if ( tag->title )
    {
        string  = g_strconcat("title=", tag->title, NULL);
        string1 = convert_to_utf8(string);
        vorbis_comment_add(vc, string1);
        g_free(string);
        g_free(string1);
    }

// Artist
    if ( tag->artist )
    {
        string  = g_strconcat("artist=", tag->artist, NULL);
        string1 = convert_to_utf8(string);
        vorbis_comment_add(vc, string1);
        g_free(string);
        g_free(string1);
    }

// Album
    if ( tag->album )
    {
        string  = g_strconcat("album=", tag->album, NULL);
        string1 = convert_to_utf8(string);
        vorbis_comment_add(vc, string1);
        g_free(string);
        g_free(string1);
    }

// Year
    if ( tag->year )
    {
        string  = g_strconcat("date=", tag->year, NULL);
        string1 = convert_to_utf8(string);
        vorbis_comment_add(vc, string1);
        g_free(string);
        g_free(string1);
    }

// Track
    if ( tag->track )
    {
        string = g_strconcat("tracknumber=", tag->track, NULL);
        string1 = convert_to_utf8(string);
        vorbis_comment_add(vc, string1);
        g_free(string);
        g_free(string1);
    }

// Genre
    if ( tag->genre )
    {
        string  = g_strconcat("genre=", genrelist[tag->genre], NULL);
        string1 = convert_to_utf8(string);
        vorbis_comment_add(vc, string1);
        g_free(string);
        g_free(string1);
    }

// Comment
// We write the comment using the "both" format
    if ( tag->comment )
    {
        string = g_strconcat("comment=", tag->comment, NULL);
        string1 = convert_to_utf8(string);
        vorbis_comment_add(vc, string1);
        g_free(string);
        g_free(string1);
    }
	//{gint i; for (i = 0; i < vc->comments; i++) g_print("%s -> ogg vc< '%s'\n",g_basename(filename_in),vc->user_comments[i]);}

// Write tag
    if ( (i = Ogg_Tag_Write_File(oggfile, filename, state)) != 0 )
    {
		g_print("ERROR: Failed to write comments to file '%s' (ERR: %i).\n", filename, i);
		return(3);
    }

    vcedit_clear(state);

    return(0);
}


int
del_vorbis_tag(gchar *filename)
{
	FILE *oggfile;
    vcedit_state *state;
    vorbis_comment *vc;
	gint i = 0;

// Open file
	oggfile = fopen(filename, "rb");
	if (!oggfile)
		return(1);

    state = vcedit_new_state();    // Allocate memory for 'state'
    if ( vcedit_open(state, oggfile) < 0 )
    {
        g_print("ERROR: Failed to open file: '%s' as vorbis (%s).\n", filename, vcedit_error(state));
        fclose(oggfile);
        return(2);
    }

// Get data from tag
    vc = vcedit_comments(state);
    vorbis_comment_clear(vc);
    vorbis_comment_init(vc);
	
// Write tag
    if ( (i = Ogg_Tag_Write_File(oggfile, filename, state)) != 0 )
    {
		g_print("ERROR: Failed to write comments to file '%s' (ERR: %i).\n", filename, i);
		return(3);
    }

    vcedit_clear(state);

    return(0);
}

#endif
