/* the gmp3 project.
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com/gmp3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "interface.h"			/* contains functions to create the gui */
#include "callbacks.h"			/* any callback funcs */
#include "support.h"			/* the glade support file, e.g. with "lookup_widget" */
#include "gui_dirtree.h"		/* Needed to init the directory tree */
#include "memory.h"				/* Defines easier/(more secure) stackhandling funcs */
#include "queue.h"				/* To init the queue */
#include "configfile.h"			/* To init the configfile */
#include "shared.h"				/* For "configfile_show_hidden" */
#include "gui_filelist.h"		/* To init the filelist */

/* Init global variables for the windows, popups and dialogs */
GtkWidget *splash = NULL;
GtkWidget *gmp3 = NULL;
GtkWidget *options = NULL;
GtkWidget *rules = NULL;
GtkWidget *popup_gmp3_queue = NULL;
GtkWidget *popup_gmp3_filelist = NULL;
GtkWidget *popup_gmp3_dirtree = NULL;
GtkWidget *popup_gmp3_log_log = NULL;
GtkWidget *popup_rename_file = NULL;
GtkWidget *popup_rename_directory = NULL;
GtkWidget *popup_create_directory = NULL;
GtkWidget *popup_really_do = NULL;
GtkWidget *custom_find = NULL;
GtkWidget *custom_view = NULL;
GtkWidget *about = NULL;
GtkWidget *album_choose = NULL;
GtkWidget *tracks_sort = NULL;
GtkWidget *wait = NULL;

/* The stacks, for a better memoryhandling */
GList *ctree_gmp3_directories_data = NULL;
GList *queue = NULL;
GList *rulelist = NULL;

/* Other globals */
gchar *current_dir = NULL;					/* Always will contain the directory name which is active for the filelist */
gchar *active_tag = NULL;					/* Will contain the full filename of the tag currently shown in the tageditor */
gint lastclick_x, lastclick_y;				/* Last click, is needed to determine the clickposition for popupmenus */
gint defaulttag = 3;						/* The default tag to view and edit for the tageditor tool */
GtkWidget *lastclicked = NULL;

/**********************************************************************
 * This is the root of all evil :)
 * Arguments: Don't matter to me here.
 * Returns: 0 if success, 1 if error.
 **********************************************************************/
int
main (int argc, char *argv[])
{
	gchar configfilename[2048] = "\0";
	gchar value[4096] = "\0";
	gchar windowtitle[1000] = "\0";
	gchar *config = malloc(10000);
	
#ifdef ENABLE_NLS
	bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
	textdomain (PACKAGE);
#endif

	current_dir = calloc(2048, sizeof(gchar));
	strncpy(current_dir, "/", 1);
	active_tag = calloc(2048, sizeof(gchar));
	
// Read configfile version;
	config_read(config, 10000);
	configfile_option_get(config, "program internal", "cfgversion", value);
	
// If we cannot access the configfile, or if it has the wrong version, create a new one or quit
	snprintf(configfilename, 2047, "%s%s", getenv("HOME"), "/.gmp3rc");
	if( access(configfilename, W_OK) != 0
		&& !configfile_set_default() )
	{
		printf("Unable to access/create configfile! (%s)\n", configfilename);
		free(config);
		exit(1);
	}
	else if( strcmp(value, "3")!=0 )
	{
		gchar *backupcfgfile = NULL;
		
		printf("Configfile is incompatible... probably out of date. Current version is: 3\nCreating new configfile...\n");
		backupcfgfile = g_strconcat(configfilename, ".old", NULL);
		if( rename(configfilename, backupcfgfile) != 0 )
		{
			printf("Couldn't backup old configfile! (New name: %s)\nExiting.\n", backupcfgfile);
			free(backupcfgfile);
			free(config);
			exit(1);
		}
		else
			printf("Created backup configfile: %s\n", backupcfgfile);
		free(backupcfgfile);
		
		if( !configfile_set_default() )
		{
			printf("Unable to create new configfile! (%s)\n", configfilename);
			free(config);
			exit(1);
		}
	}

// gtk init
	gnome_init ("gmp3", VERSION, argc, argv);

// Create widgets
	popup_gmp3_queue = create_popup_gmp3_queue();
	popup_gmp3_filelist = create_popup_gmp3_filelist();
	popup_gmp3_dirtree = create_popup_gmp3_dirtree();
	popup_gmp3_log_log = create_popup_gmp3_log_log();
	gmp3 = create_gmp3 ();
	splash = create_splash();
	wait = create_wait();

#ifndef HAVE_OGG_H
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(gmp3, "radio_gmp3_tagedit_both")), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(gmp3, "radio_gmp3_tagedit_ogg")), FALSE);
#endif

// set window title
	sprintf(windowtitle, "gmp3 %s", VERSION);
	gtk_window_set_title(GTK_WINDOW(gmp3), windowtitle);
	
// Determine defaulttag for the tageditor
	defaulttag = 3;
	configfile_option_get(config, "program general", "default tag v1", value);
	if( strcmp(value, "TRUE")==0 )
		defaulttag = 1;
	configfile_option_get(config, "program general", "default tag v2", value);
	if( strcmp(value, "TRUE")==0 )
		defaulttag = 2;

// Show splashscreen, if enabled
	configfile_option_get(config, "program general", "hide splash", value);
	if(strncmp(value, "FALSE", 5)==0)
	{
		gtk_widget_show (splash);
		gtk_timeout_add(3000, on_splash_timeout, NULL);
	}
	else
		gtk_widget_show (gmp3);

// This will set correct column titles to the queue lists.
	queue_update(GTK_CLIST(lookup_widget(gmp3, "clist_gmp3_queue")), TRUE);
	queue_update(GTK_CLIST(lookup_widget(gmp3, "clist_gmp3_queueonly_queue")), TRUE);
	
// init dirtree
	ctree_init_dirtree(
		GTK_CTREE(lookup_widget(gmp3, "ctree_gmp3_directories")),
		ctree_gmp3_directories_data);
	
// open the homedir or, if specified, the default directory
	configfile_option_get(config, "program general", "default directory", value);
	free(config);
	if(!value || *value == '\0')
		strncpy(value, getenv("HOME"), 4096);
	if(value && *value != '\0')
	{
		ctree_dirtree_open_directory(
			GTK_CTREE(lookup_widget(gmp3, "ctree_gmp3_directories")),
			value);
		strncpy(current_dir, value, 2047);
		clist_filelist_update(GTK_CLIST(lookup_widget(gmp3, "clist_gmp3_filelist")),
			current_dir,
			config_show_hidden() );
	}

	gtk_main ();
	
	free(current_dir);
	free(active_tag);
	
	ctree_gmp3_directories_data = mem_free(ctree_gmp3_directories_data);
	queue = mem_free(queue);
	rulelist = mem_free(rulelist);
	
	return 0;
}
