/* the gmp3 project.
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com/gmp3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "support.h"
#include "memory.h"

#include "queue.h"
#include "configfile.h"
#include "gui_filelist.h"
#include "gui_queue.h"
#include "mp3info.h"
#include "gui_shared.h"
#include "shared.h"

/***************************************************************************************
 * BELOW FOLLOW THE STATICS
 ***************************************************************************************/

// determine the correct order for the queue columns
static void
get_column_order(gchar *name, gchar *order[])
{
	gchar option[2048];
	gchar value[2048];
	gint cur = 0;
	gchar *config = malloc(10000);
	
// read configfile
	config_read(config, 10000);
	
	order[0] = NULL;
	snprintf(option, 2047, "column0#1");
// get correct order from configfile
	while(configfile_option_get(config, name, option, value))
	{
		order[cur] = strdup(value);
		snprintf(option, 2047, "column%i#1", ++cur);
		order[cur] = NULL;
	}
	free(config);
}


// Removes all items from haystack that have a filename that is in needle.
static GList *
g_list_remove_matching_filenames(GList *haystack, GList *needle)
{
	GList *haystack_item = NULL;
	GList *needle_item = NULL;
	Mp3Info *haystack_mp3info = NULL;
	Mp3Info *needle_mp3info = NULL;

	haystack_item = g_list_first(haystack);
	while(haystack_item)
	{
		haystack_mp3info = (Mp3Info *)haystack_item->data;
		haystack_item = g_list_next(haystack_item);
		
		needle_item = g_list_first(needle);
		while(needle_item)
		{
			needle_mp3info = (Mp3Info *)needle_item->data;
			needle_item = g_list_next(needle_item);
			if(strncmp(needle_mp3info->filename, haystack_mp3info->filename, 2048)==0)
			{
				haystack = mem_free_1(haystack, haystack_mp3info);
				break;
			}
		}
	}
	return g_list_first(haystack);
}

/***************************************************************************************
 * END OF STATICS
 ***************************************************************************************/





/**********************************************************************
 * Adds all selected items in the filelist to a queue list
 **********************************************************************/
void
queue_files_add_selected(void)
{
	extern GtkWidget *gmp3;
	extern gchar *current_dir;
	extern GList *queue;
	
	GList *filelist = NULL;
	GList *queuelist = NULL;
	gchar *order[30];
	gint cur = 0;
	
	gchar *config = malloc(10000);
	gchar toggle[2048];

// read configfile
	config_read(config, 10000);
	configfile_option_get(config, "program general", "toggle queue background", toggle);
	free(config);
	
// Get a list of all filenames selected
	filelist = clist_filenames_get_selected(GTK_CLIST(lookup_widget(gmp3, "clist_gmp3_filelist")), current_dir);
// Get all tag and mpeg infos (and other) for that file
	queuelist = mp3file_get_infos(filelist);
// Free the filename list and its data
	mem_free(filelist);
	
// get the wished column style/order from the configfile
	get_column_order("queue columns", order);
// set the column titles
	clist_queue_set_titles(GTK_CLIST(lookup_widget(gmp3, "clist_gmp3_queue")), order);
	clist_queue_set_titles(GTK_CLIST(lookup_widget(gmp3, "clist_gmp3_queueonly_queue")), order);
	
// Remove items from the list which are already in the queue.
	queuelist = g_list_remove_matching_filenames(queuelist, queue);

// Add the desired data to the queue (both)
	clist_queue_add_files(GTK_CLIST (lookup_widget (gmp3, "clist_gmp3_queue")), &queuelist, order);
	clist_queue_add_files(GTK_CLIST (lookup_widget (gmp3, "clist_gmp3_queueonly_queue")), &queuelist, order);
	queue = g_list_concat(queue, queuelist);

// set every second row to grey background
	if(strncmp(toggle, "TRUE", 4)==0)
	{
		clist_queue_toggle_colors(GTK_CLIST(lookup_widget (gmp3, "clist_gmp3_queue")));
		clist_queue_toggle_colors(GTK_CLIST(lookup_widget (gmp3, "clist_gmp3_queueonly_queue")));
	}

	cur = -1;
	while(order[++cur])
		free(order[cur]);	
	return;
}







/**********************************************************************
 * Removes all files from the queue clist and adds them again
 **********************************************************************/
void
queue_update(GtkCList *clist, gboolean reset)
{
	extern GList *queue;
	gchar *order[30];
	gint cur = 0;
	gchar *config = malloc(10000);
	gchar toggle[2048];

// read configfile
	config_read(config, 10000);
	configfile_option_get(config, "program general", "toggle queue background", toggle);
	free(config);
	
	gtk_clist_freeze(clist);

// empty lists out.
	gtk_clist_clear(clist);
	
// get the wished column style/order from the configfile
	get_column_order("queue columns", order);
	
// set the column titles
	if(reset)
		clist_queue_set_titles(clist, order);
		
// Add the desired data to the queue (both)
	clist_queue_add_files(clist, &queue, order);
	
// set every second row to grey background
	if(strncmp(toggle, "TRUE", 4)==0)
		clist_queue_toggle_colors(clist);
	
	gtk_clist_thaw(clist);

	cur = -1;
	while(order[++cur])
		free(order[cur]);	
	return;
}





/**********************************************************************
 * Empties out all queue lists and frees the allocated memory
 **********************************************************************/
void
queue_clear(void)
{
	extern GtkWidget *gmp3;
	extern GList *queue;

// empty lists out.
	gtk_clist_clear(GTK_CLIST(lookup_widget(gmp3, "clist_gmp3_queue")));
	gtk_clist_clear(GTK_CLIST(lookup_widget(gmp3, "clist_gmp3_queueonly_queue")));
	queue = mem_free(queue);
}





/**********************************************************************
 * Removes all selected items from the queue and from the gui clist
 * Arguments: A queue clist.
 **********************************************************************/
void
queue_files_remove_selected(GtkCList *clist)
{
	extern GtkWidget *gmp3;
	extern GList *queue;
	
	GList *selected = NULL;
	gchar *order[30];
	gint cur = 0;
	
	gchar *config = malloc(10000);
	gchar toggle[2048];

// read configfile
	config_read(config, 10000);
	configfile_option_get(config, "program general", "toggle queue background", toggle);
	free(config);
	
// Get a list of all filenames selected
	selected = clist_get_selected(clist);
	
// get the wished column style/order from the configfile
	get_column_order("queue columns", order);
// set the column titles
	clist_queue_set_titles(GTK_CLIST(lookup_widget(gmp3, "clist_gmp3_queue")), order);
	clist_queue_set_titles(GTK_CLIST(lookup_widget(gmp3, "clist_gmp3_queueonly_queue")), order);
	
// Remove items from the list which are already in the queue.
	clist_files_remove(GTK_CLIST(lookup_widget(gmp3, "clist_gmp3_queue")), selected);
	clist_files_remove(GTK_CLIST(lookup_widget(gmp3, "clist_gmp3_queueonly_queue")), selected);
	queue = g_list_remove_matching_filenames(queue, selected);
	g_list_free(selected);

// set every second row to grey background
	if(strncmp(toggle, "TRUE", 4)==0)
		clist_queue_toggle_colors(clist);
	
	cur = -1;
	while(order[++cur])
		free(order[cur]);	
	return;
}









/**********************************************************************
 * Adds files recursive to a queue list, beginning at current_dir
 **********************************************************************/
void
queue_files_add_recursive(gchar *current_dir)
{
	extern GtkWidget *gmp3;
	extern GList *queue;
	
	GList *filelist = NULL;
	GList *filelist2 = NULL;
	GList *queuelist = NULL;
	gchar *order[30];
	gint cur = 0;
	
	gchar *config = malloc(10000);
	gchar toggle[2048];

// read configfile
	config_read(config, 10000);
	configfile_option_get(config, "program general", "toggle queue background", toggle);
	free(config);
	
	gnome_appbar_set_status(GNOME_APPBAR(lookup_widget(gmp3, "appbar")), "Prescanning directories...");
	while(g_main_iteration(FALSE));

// Get a list of all filenames ("*.mp3") in currentdir and its subs. CASE INSENSITIVE.
	filelist = get_files_recursive(current_dir, "*.mp3");
	if( (filelist2 = get_files_recursive(current_dir, "*.Mp3")) )
		filelist = g_list_concat(filelist, filelist2);
	if( (filelist2 = get_files_recursive(current_dir, "*.mP3")) )
		filelist = g_list_concat(filelist, filelist2);
	if( (filelist2 = get_files_recursive(current_dir, "*.MP3")) )
		filelist = g_list_concat(filelist, filelist2);
#ifdef HAVE_OGG_H
	if( (filelist2 = get_files_recursive(current_dir, "*.ogg")) )
		filelist = g_list_concat(filelist, filelist2);
	if( (filelist2 = get_files_recursive(current_dir, "*.Ogg")) )
		filelist = g_list_concat(filelist, filelist2);
	if( (filelist2 = get_files_recursive(current_dir, "*.oGg")) )
		filelist = g_list_concat(filelist, filelist2);
	if( (filelist2 = get_files_recursive(current_dir, "*.ogG")) )
		filelist = g_list_concat(filelist, filelist2);
	if( (filelist2 = get_files_recursive(current_dir, "*.OGg")) )
		filelist = g_list_concat(filelist, filelist2);
	if( (filelist2 = get_files_recursive(current_dir, "*.oGG")) )
		filelist = g_list_concat(filelist, filelist2);
	if( (filelist2 = get_files_recursive(current_dir, "*.OgG")) )
		filelist = g_list_concat(filelist, filelist2);
	if( (filelist2 = get_files_recursive(current_dir, "*.OGG")) )
		filelist = g_list_concat(filelist, filelist2);
#endif

// Get all tag and mpeg infos (and other) for that filenames
	queuelist = mp3file_get_infos(filelist);
// Free the filename list and its data
	mem_free(filelist);
	
// get the wished column style/order from the configfile
	get_column_order("queue columns", order);
// set the column titles
	clist_queue_set_titles(GTK_CLIST(lookup_widget(gmp3, "clist_gmp3_queue")), order);
	clist_queue_set_titles(GTK_CLIST(lookup_widget(gmp3, "clist_gmp3_queueonly_queue")), order);
	
// Remove items from the list which are already in the queue.
	queuelist = g_list_remove_matching_filenames(queuelist, queue);

// Add the desired data to the queue (both)
	clist_queue_add_files(GTK_CLIST(lookup_widget (gmp3, "clist_gmp3_queue")), &queuelist, order);
	clist_queue_add_files(GTK_CLIST(lookup_widget (gmp3, "clist_gmp3_queueonly_queue")), &queuelist, order);
	queue = g_list_concat(queue, queuelist);
	
// set every second row to grey background
	if(strncmp(toggle, "TRUE", 4)==0)
	{
		clist_queue_toggle_colors(GTK_CLIST(lookup_widget (gmp3, "clist_gmp3_queue")));
		clist_queue_toggle_colors(GTK_CLIST(lookup_widget (gmp3, "clist_gmp3_queueonly_queue")));
	}

	cur = -1;
	while(order[++cur])
		free(order[cur]);	
	return;
}





