/* the gmp3 project.
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com/gmp3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "support.h"

#include "lib_mpgheader.h"
#include "lib_id3v1.h"
#include "lib_id3v2.3.h"
#include "tageditor.h"
#include "gui_tageditor.h"
#include "mp3info.h"
#include "lib_vorbis.h"


/***************************************************************************************
 * BELOW FOLLOW THE STATICS
 ***************************************************************************************/

// Returns the datapointer to the queueitems data with the given fullfilename
static Mp3Info *
g_list_find_matching_fullfilename(GList *haystack, gchar *fullfilename)
{
	GList *haystack_item = NULL;
	Mp3Info *haystack_mp3info = NULL;
	gchar haystack_fullfilename[2048];

	haystack_item = g_list_first(haystack);
	while(haystack_item)
	{
		haystack_mp3info = (Mp3Info *)haystack_item->data;
		haystack_item = g_list_next(haystack_item);
		
		snprintf(haystack_fullfilename, 2047, "%s%s", haystack_mp3info->directory, haystack_mp3info->filename);
		if(strncmp(fullfilename, haystack_fullfilename, 2048)==0)
			return haystack_mp3info;
	}
	return NULL;
}

/***************************************************************************************
 * END OF STATICS
 ***************************************************************************************/








/**********************************************************************
 * this will update all informations in the tageditor gui
 * (tags content & mp3header info).
 **********************************************************************/
void tagedit_info_update(gchar *filename, gint whichtag, gint mode)
{
	extern GtkWidget *gmp3;
	Header *mp3header = malloc(sizeof(Header));
	gchar foo[2000];
	gboolean have_tag = FALSE;
	
	id3Tag *tag = calloc(1, sizeof(id3Tag));
	id3Tag *v2 = malloc(sizeof(id3Tag));
	
	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, "appbar")), "Reading tag...");
	if(get_mpgheader(mp3header, filename)==0)
	{
		sprintf(foo, "MPEG V%s, Layer %i\nSample rate: %i\nBitrate: %i\n",
			mp3header->version, mp3header->layer, mp3header->frequency, mp3header->bitrate);
		if(mp3header->seconds-( ((int)mp3header->seconds/60)*60 )<10)
		{
			sprintf(foo, "%sTime: %i:0%i\n", foo,
				(int)mp3header->seconds/60, mp3header->seconds-( ((int)mp3header->seconds/60)*60 ));
		}
		else
		{
			sprintf(foo, "%sTime: %i:%i\n", foo,
				(int)mp3header->seconds/60, mp3header->seconds-( ((int)mp3header->seconds/60)*60 ));
		}
		if(mp3header->copyright)
			sprintf(foo, "%sCopyright: Yes\n", foo);
		else
			sprintf(foo, "%sCopyright: No\n", foo);
		if(mp3header->original)
			sprintf(foo, "%sOriginal/copy: Original\n", foo);
		else
			sprintf(foo, "%sOriginal/copy: Copy\n", foo);
			
		if(get_id3v1_tag(tag, filename)==0)
			sprintf(foo, "%sID3V1 Tag present.\n", foo);
			
		if(get_id3v2_tag(tag, filename)==0)
			sprintf(foo, "%sID3V2 Tag present.\n", foo);
		gtk_label_set_text(GTK_LABEL (lookup_widget (gmp3, "label_gmp3_tagedit_mpginfo")), foo);
	}
	else
	{
		gtk_label_set_text(GTK_LABEL (lookup_widget (gmp3, "label_gmp3_tagedit_mpginfo")), "No valid header found!");
	}
	
// first of all, empty all out (only if in normal view mode,
// otherwise we have to keep all those which are the same in the previous tag)
	if(mode==0)
	{
		gtk_entry_set_text(GTK_ENTRY (lookup_widget (gmp3, "entry_gmp3_tagedit_artist")), (char *)"");
		gtk_entry_set_text(GTK_ENTRY (lookup_widget (gmp3, "entry_gmp3_tagedit_song")), (char *)"");
		gtk_entry_set_text(GTK_ENTRY (lookup_widget (gmp3, "entry_gmp3_tagedit_album")), (char *)"");
		gtk_entry_set_text(GTK_ENTRY (lookup_widget (gmp3, "entry_gmp3_tagedit_year")), (char *)"");
		gtk_entry_set_text(GTK_ENTRY (lookup_widget (gmp3, "entry_gmp3_tagedit_track")), (char *)"");
		gtk_entry_set_text(GTK_ENTRY (lookup_widget (gmp3, "entry_gmp3_tagedit_genre")), "Other");
		gtk_text_set_point(GTK_TEXT (lookup_widget (gmp3, "text_gmp3_tagedit_comment")), 0);
		gtk_text_forward_delete(GTK_TEXT (lookup_widget (gmp3, "text_gmp3_tagedit_comment")), 
			gtk_text_get_length(GTK_TEXT (lookup_widget (gmp3, "text_gmp3_tagedit_comment"))));
	}
	
// get the tag to a struct.
	if( (whichtag==1 || whichtag==3) && get_id3v1_tag(tag, filename)==0)
		have_tag = TRUE;
// v2 tag has higher prio, so it will overwrite the v1	
	if( (whichtag==2 || whichtag==3) && get_id3v2_tag(v2, filename)==0)
	{
		have_tag = TRUE;
		v2->genre = tag->genre;
		free(tag);
		tag = v2;
	}
	else
		free(v2);
#ifdef HAVE_OGG_H
	if( whichtag==4 && get_vorbis_tag(tag, filename)==0 )
		have_tag = TRUE;
#endif
	
	if(!have_tag)
	{
		free(tag);
		free(mp3header);
		gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, "appbar")), "");
		return;
	}

// and put the tag to the form
	tageditor_tag_to_gui(tag, mode);

	free(tag);
	free(mp3header);
	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, "appbar")), "");
	return;
}


















/**********************************************************************
 * this will write all informations in the tageditor gui to one tag
 **********************************************************************/
void
tagedit_tag_set(gchar *filename, gint whichtag)
{
	extern GtkWidget *gmp3;
	extern GList *queue;
	
	id3Tag *tag = malloc(sizeof(id3Tag));
	gchar bar[4096];
	Mp3Info *mp3info = NULL;

	if(!filename || *filename == '\0')
	{
		free(tag);
		return;
	}
	
	sprintf(bar, "Saving tags: %s", filename);
	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, "appbar")), bar);
	while(g_main_iteration(FALSE));
	
// Get old Tag
	memset(tag->artist, '\0', 1024);
	memset(tag->title, '\0', 1024);
	memset(tag->album, '\0', 1024);
	memset(tag->year, '\0', 1024);
	memset(tag->comment, '\0', 1024);
	memset(tag->track, '\0', 3);

// get the tag to a struct.
	if( (whichtag==1 || whichtag==3) )
		get_id3v1_tag(tag, filename);
// if the v2 tag should be set, overwrite the v1 struct (exept the genre!)
	if( (whichtag==2 || whichtag==3) )
		get_id3v2_tag(tag, filename);
#ifdef HAVE_OGG_H
	if(whichtag==4)
		get_vorbis_tag(tag, filename);
#endif

// get tag info from gui
	tageditor_gui_to_tag(tag);
	
	mp3info = g_list_find_matching_fullfilename(queue, filename);

// And put the changed tag to the file
	if(access(filename, W_OK)==0)
	{
		if(queue)
			strncpy(mp3info->status, "Tags set!", 255);
		if( strcmp(filename+strlen(filename)-4, ".ogg")==0
			|| strcmp(filename+strlen(filename)-4, ".Ogg")==0
			|| strcmp(filename+strlen(filename)-4, ".oGg")==0
			|| strcmp(filename+strlen(filename)-4, ".ogG")==0
			|| strcmp(filename+strlen(filename)-4, ".OGg")==0
			|| strcmp(filename+strlen(filename)-4, ".oGG")==0
			|| strcmp(filename+strlen(filename)-4, ".OgG")==0
			|| strcmp(filename+strlen(filename)-4, ".OGG")==0 )
		{
#ifdef HAVE_OGG_H
			if( whichtag==4 && set_vorbis_tag(tag, filename)!=0 && queue)
				strncpy(mp3info->status, "Error!", 255);
#endif
		}
		else
		{
			if( (whichtag==1 || whichtag==3) && set_id3v1_tag(tag, filename)!=0 && queue)
				strncpy(mp3info->status, "Error!", 255);
			if( (whichtag==2 || whichtag==3) && set_id3v2_tag(tag, filename)!=0 && queue)
				strncpy(mp3info->status, "Error!", 255);
		}
	}
	else
		if(queue)
			strncpy(mp3info->status, "Locked!", 255);

	free(tag);
	
// Update the taginfo in queue (not in the gui!)
	strncpy(bar, filename, 2047);
	*(strrchr(bar, '/')+1) = '\0';
	mp3file_get_info_1(bar, strrchr(filename, '/')+1, mp3info);
	
	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, "appbar")), "Ready.");
}










/**********************************************************************
 * this will write all informations in the tageditor gui to all tags in queue
 **********************************************************************/
void
tagedit_tag_set_queue(gint whichtag)
{
	extern GList *queue;
	GList *item = NULL;
	Mp3Info *mp3info = NULL;
	gchar fullfilename[2048];
	
	item = g_list_first(queue);
	while(item)
	{
		mp3info = (Mp3Info *)item->data;
		snprintf(fullfilename, 2047, "%s%s", mp3info->directory, mp3info->filename);
		tagedit_tag_set(fullfilename, whichtag);
		item = item->next;
	}
}
















/**********************************************************************
 * this will delete one tag
 **********************************************************************/
void
tagedit_tag_delete(gchar *filename, gint whichtag)
{
	extern GtkWidget *gmp3;
	extern GList *queue;
	
	gchar bar[4096];

	if(!filename || *filename == '\0')
		return;
	
	sprintf(bar, "Deleting tags: %s", filename);
	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, "appbar")), bar);
	while(g_main_iteration(FALSE));
		
// And put the changed tag to the file
	if(access(filename, W_OK)==0)
	{
		if( strcmp(filename+strlen(filename)-4, ".ogg")==0
			|| strcmp(filename+strlen(filename)-4, ".Ogg")==0
			|| strcmp(filename+strlen(filename)-4, ".oGg")==0
			|| strcmp(filename+strlen(filename)-4, ".ogG")==0
			|| strcmp(filename+strlen(filename)-4, ".OGg")==0
			|| strcmp(filename+strlen(filename)-4, ".oGG")==0
			|| strcmp(filename+strlen(filename)-4, ".OgG")==0
			|| strcmp(filename+strlen(filename)-4, ".OGG")==0 )
		{
#ifdef HAVE_OGG_H
			if(whichtag==4)
			{
				del_vorbis_tag(filename);
			}
#endif
		}
		else
		{
			if( (whichtag==1 || whichtag==3) )
				del_id3v1_tag(filename);
			if( (whichtag==2 || whichtag==3) )
				del_id3v2_tag(filename);
		}
	}

// Update the taginfo in queue (not in the gui!)
	strncpy(bar, filename, 2047);
	*(strrchr(bar, '/')+1) = '\0';
	mp3file_get_info_1(bar, strrchr(filename, '/')+1,
		g_list_find_matching_fullfilename(queue, filename));
	gnome_appbar_set_status(GNOME_APPBAR (lookup_widget (gmp3, "appbar")), "Ready.");
}













/**********************************************************************
 * this will delete all tags of the files in queue
 **********************************************************************/
void
tagedit_tag_delete_queue(gint whichtag)
{
	extern GList *queue;
	GList *item = NULL;
	Mp3Info *mp3info;
	gchar fullfilename[2048];
	
	item = g_list_first(queue);
	while(item)
	{
		mp3info = (Mp3Info *)item->data;
		snprintf(fullfilename, 2047, "%s%s", mp3info->directory, mp3info->filename);
		tagedit_tag_delete(fullfilename, whichtag);
		item = item->next;
	}
}
